@@ -15,8 +15,6 @@ import json
 import logging
 import os
 import signal
-from collections import deque
-from threading import Lock
 from typing import Callable, Dict, List, Optional, Tuple
 
 from .command import Command
@@ -26,6 +24,8 @@ from .message import Message, process_msg
 from .packet import set_pkt_logging
 from .protocol import create_msg_stack
 from .schema import (
+    ALLOW_LIST,
+    BLOCK_LIST,
     DEBUG_MODE,
     DISABLE_DISCOVERY,
     DISABLE_SENDING,
@@ -87,12 +87,9 @@ class Gateway:
         if self.config[REDUCE_PROCESSING] < DONT_CREATE_MESSAGES:
             self.msg_protocol, self.msg_transport = self.create_client(process_msg)
 
-        self._buffer = deque()
-        self._sched_zone = None
-        self._sched_lock = Lock()
-
-        # if config.get("ser2net_server"):
-        self._relay = None  # ser2net_server relay
+        # self._buffer = deque()
+        # self._sched_zone = None
+        # self._sched_lock = Lock()
 
         # if self.config[REDUCE_PROCESSING] > 0:
         self.rfg = None
@@ -202,9 +199,22 @@ class Gateway:
                 system._discover()  # discover_flag=DISCOVER_ALL)
             return system
 
-        def create_device(dev_addr) -> Device:
+        def create_device(dev_addr) -> Device:  # TODO: Optional[Device]
             # assert dev_addr.id not in self.device_by_id, f"Dup. dev_id: {dev_addr.id}"
 
+            if self._include and dev_addr.id not in self._include:
+                _LOGGER.warning(
+                    f"Creating a non-allowed device_id: {dev_addr.id}"
+                    f" (consider addding it to the {ALLOW_LIST})"
+                )
+
+            elif dev_addr.id in self._exclude:
+                _LOGGER.warning(
+                    f"Creating a blocked device_id: {dev_addr.id}"
+                    f" (consider removing it from the {BLOCK_LIST})"
+                )
+
+            # else:
             device = DEVICE_CLASSES.get(dev_addr.type, Device)(self, dev_addr)
 
             # if isinstance(device, Controller):
@@ -234,11 +244,11 @@ class Gateway:
             self.rfg = dev
 
         # update the existing device with any metadata
-        if ctl_addr is not None:
+        if ctl_addr and ctl:
             dev._set_ctl(ctl)
         if domain_id in ("F9", "FA", "FC", "FF"):
             dev._domain_id = domain_id
-        elif domain_id is not None and ctl_addr is not None:
+        elif domain_id is not None and ctl_addr and ctl:
             dev._set_zone(ctl._evo._get_zone(domain_id))
 
         return dev
@@ -363,7 +373,7 @@ class Gateway:
     async def async_send_cmd(
         self, cmd: Command, awaitable: bool = True, **kwargs
     ) -> Optional[Message]:
-        """Send a command with the option to return any response.
+        """Send a command with the option to not wait for a response (awaitable=False).
 
         Response packets, if any, follow an RQ/W (as an RP/I), and have the same code.
         """
