@@ -40,7 +40,7 @@ DEFAULT_EXT_ID = "17:000730"
 DEFAULT_THM_ID = "03:000730"
 
 
-DEV_MODE = __dev_mode__  # and False
+DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -786,13 +786,17 @@ class UfhController(Device):  # UFC (02):
 
     @property
     def circuits(self) -> Optional[Dict]:  # 000C
-        def fix(k):
-            return "zone_idx" if k == "zone_id" else k
+        return {
+            k: {"zone_idx": m.payload["zone_id"]} for k, m in self._circuits.items()
+        }
 
-        return [
-            {fix(k): v for k, v in m.payload.items() if k in ("ufh_idx", "zone_id")}
-            for m in self._circuits.values()
-        ]
+        # def fix(k):
+        #     return "zone_idx" if k == "zone_id" else k
+
+        # return [
+        #     {fix(k): v for k, v in m.payload.items() if k in ("ufh_idx", "zone_id")}
+        #     for m in self._circuits.values()
+        # ]
 
     @property
     def heat_demand(self) -> Optional[float]:  # 3150
@@ -808,10 +812,14 @@ class UfhController(Device):  # UFC (02):
 
     @property
     def setpoints(self) -> Optional[Dict]:  # 22C9
-        return [
-            {k: v for k, v in d.items() if k in ("ufh_idx", "temp_high", "temp_low")}
-            for d in (self._setpoints.payload if self._setpoints else [])
-        ]
+        return {
+            c["ufh_idx"]: {"temp_high": c["temp_high"], "temp_low": c["temp_low"]}
+            for c in self._setpoints.payload
+        }
+        # return [
+        #     {k: v for k, v in d.items() if k in ("ufh_idx", "temp_high", "temp_low")}
+        #     for d in (self._setpoints.payload if self._setpoints else [])
+        # ]
 
     @property  # id, type
     def schema(self) -> dict:
@@ -824,7 +832,7 @@ class UfhController(Device):  # UFC (02):
     def params(self) -> dict:
         return {
             **super().params,
-            "setpoints": self.setpoints,
+            "circuits": self.setpoints,
         }
 
     @property
@@ -1336,17 +1344,17 @@ class FanDevice(Device):  # FAN (20/37):
     @property
     def fan_rate(self) -> Optional[float]:
         msgs = [m for m in self._msgs.values() if m.code in ("31D9", "31DA")]
-        return max(msgs).payload[self.FAN_RATE] if msgs else None
+        return max(msgs).payload["exhaust_fan_speed"] if msgs else None
 
     @property
     def boost_timer(self) -> Optional[int]:
         if "31DA" in self._msgs:
-            return self._msgs["31DA"].payload[self.BOOST_TIMER]
+            return self._msgs["31DA"].payload["remaining_time"]
 
     @property
     def relative_humidity(self) -> Optional[float]:
         if "31DA" in self._msgs:
-            return self._msgs["31DA"].payload[self.RELATIVE_HUMIDITY]
+            return self._msgs["31DA"].payload["indoor_humidity"]
 
     @property
     def status(self) -> dict:
