@@ -62,7 +62,7 @@ IDX_NAMES = {
 I_, RQ, RP, W_ = " I", "RQ", "RP", " W"
 
 DEV_MODE = __dev_mode__
-TEST_MODE = False
+TEST_MODE = False  # enable to test constructors (usu. W)
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -77,7 +77,7 @@ def re_compile_re_match(regex, string) -> bool:
 @lru_cache(maxsize=128)
 def _idx_idx(seqx, code, src_type, dst_type) -> dict:
 
-    if code in IDX_NAMES:
+    if code in IDX_NAMES:  # these codes always have these idx_names
         return {IDX_NAMES[code]: seqx}
 
     elif code == "0016":  # WIP, not normally {"uses_zone_idx": True}
@@ -118,7 +118,8 @@ def _idx_idx(seqx, code, src_type, dst_type) -> dict:
 
         return {}
 
-    raise TypeError(code)
+    # Consider RAMSES_CODES, or CODES_WITHOUT_IDX
+    raise TypeError(f"{src_type}:", code, seqx)
 
 
 def _idx(seqx, msg) -> dict:
@@ -317,9 +318,9 @@ def _date(value: str) -> Optional[str]:  # YY-MM-DD
 def _percent(value: str) -> Optional[float]:  # a percentage 0-100% (0.0 to 1.0)
     """Return a percentage, 0-100% with resolution of 0.5%."""
     assert len(value) == 2, "len is not 2"
-    if value in ("FE", "FF"):  # TODO: diff b/w FE (seen with 3150) & FF
+    if value in ("EF", "FE", "FF"):  # TODO: diff b/w FE (seen with 3150) & FF
         return
-    assert int(value, 16) <= 200, "max value should be C8"
+    assert int(value, 16) <= 200, "max value should be 0xC8, not 0x{value}"
     return int(value, 16) / 200
 
 
@@ -355,6 +356,15 @@ def _flag8(byte, *args) -> list:
     return ret
 
 
+def _double(val, factor=1) -> Optional[float]:
+    """Return a double, used by 31DA."""
+    if val == "7FFF":
+        return
+    result = int(val, 16)
+    assert result < 32767
+    return result if factor == 1 else result / factor
+
+
 @parser_decorator  # rf_unknown
 def parser_0001(payload, msg) -> Optional[dict]:
     # When in test mode, a 12: will send a W every 6 seconds, *on?* the second:
@@ -594,7 +604,7 @@ def parser_000a(payload, msg) -> Union[dict, list, None]:
     if msg.verb == RQ and msg.len <= 2:
         return _idx(payload[:2], msg)
 
-    if msg.is_array:  # TODO: these msgs can require 2 pkts!
+    if msg.is_array:  # or msg.len >= 6:  # TODO: these msgs can require 2 pkts!
         assert msg.len >= 6 and msg.len % 6 == 0, "expecting length mod 6"
         return [_parser(payload[i : i + 12]) for i in range(0, len(payload), 12)]
 
@@ -917,7 +927,7 @@ def parser_1030(payload, msg) -> Optional[dict]:
             "valve_run_time",
             "pump_run_time",
         )
-        cmd = Command.set_mix_valve_params(
+        cmd = Command.get_mix_valve_params(
             msg.dst.id, payload[:2], **{k: v for k, v in result.items() if k in KEYS}
         )
         assert cmd.payload == payload, cmd.payload
@@ -1008,16 +1018,64 @@ def parser_10a0(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # device_info
 def parser_10e0(payload, msg) -> Optional[dict]:
-    assert msg.len in (19, 28, 30, 36, 38), msg.len  # a non-evohome seen with 30
+    assert msg.len >= 19, msg.len  # in (19, 28, 30, 36, 38), msg.len
+
+    DEV_MODE = False
+    if not DEV_MODE:
+        pass
+    elif msg.src.type == "01":
+        assert payload[2:20] in (
+            "0002FF0119FFFFFFFF",
+            "0002FF0163FFFFFFFF",
+        ), payload[2:20]
+    elif msg.src.type == "02":
+        assert payload[2:20] in ("0003FF0203FFFF0001",), payload[2:20]
+    elif msg.src.type == "04":
+        assert payload[2:20] in (
+            "0002FF0412FFFFFFFF",
+            "0002FF050BFFFFFFFF",
+        ), payload[2:20]
+    elif msg.src.type == "08":
+        assert payload[2:20] in ("0002FF0802FFFFFFFE",), payload[2:20]
+    elif msg.src.type == "10":
+        assert payload[2:20] in (
+            "0001C8810B0700FEFF",
+            "0002FF0A0CFFFFFFFF",
+        ), payload[2:20]
+    elif msg.src.type == "20":
+        assert payload[2:20] in (
+            "000100140C06010000",
+            "0001001B190B010000",
+            "0001001B221201FEFF",
+            "0001001B271501FEFF",
+            "0001001B281501FEFF",
+        ), payload[2:20]
+    elif msg.src.type == "30":
+        assert payload[2:20] in (
+            "0001C90011006CFEFF",
+            "0002FF1E01FFFFFFFF",
+            "0002FF1E03FFFFFFFF",
+        ), payload[2:20]
+    elif msg.src.type == "31":
+        assert payload[2:20] in ("0002FF1F02FFFFFFFF",), payload[2:20]
+    elif msg.src.type == "32":
+        assert payload[2:20] in ("0001C85802016CFFFF",), payload[2:20]
+    elif msg.src.type == "34":
+        assert payload[2:20] in (
+            "0001C8380A0100F1FF",
+            "0001C8380F0100F1FF",
+        ), payload[2:20]
 
     date_2 = _date(payload[20:28])  # could be 'FFFFFFFF'
     date_1 = _date(payload[28:36])  # could be 'FFFFFFFF'
+    description = bytearray.fromhex(payload[36:]).split(b"\x00")[0].decode()
 
     return {  # TODO: add version?
-        "_unknown": payload[:20],
+        "unknown": payload[2:20],
         "date_2": date_2 if date_2 else "0000-00-00",
         "date_1": date_1 if date_1 else "0000-00-00",
-        "description": _str(payload[36:]),
+        "description": description,
+        "_unknown_2": payload[38 + len(description) * 2 :],
     }
 
 
@@ -1112,7 +1170,7 @@ def parser_1290(payload, msg) -> Optional[dict]:
 @parser_decorator  # hvac_1298
 def parser_1298(payload, msg) -> Optional[dict]:
     #  I --- 37:258565 --:------ 37:258565 1298 003 0007D0
-    return {"_unknown": _temp(payload[2:])}  # TODO: _temp is a guess
+    return {"unknown": _double(payload[2:])}
 
 
 @parser_decorator  # indoor_humidity (Nuaire RH sensor)
@@ -1155,7 +1213,8 @@ def parser_12c0(payload, msg) -> Optional[dict]:
 @parser_decorator  # hvac_12C8
 def parser_12c8(payload, msg) -> Optional[dict]:
     #  I --- 37:261128 --:------ 37:261128 12C8 003 000040
-    return {"_unknown": _percent(payload[4:])}  # TODO: _percent is a guess
+    assert payload[2:4] == "00"
+    return {"unknown": _percent(payload[4:])}
 
 
 @parser_decorator  # system_sync
@@ -1375,7 +1434,7 @@ def parser_2309(payload, msg) -> Union[dict, list, None]:
     if msg.verb == RQ and msg.len <= 2:
         return _idx(payload[:2], msg)
 
-    if msg.is_array:
+    if msg.is_array:  # or msg.len >= 6:
         assert msg.len >= 3 and msg.len % 3 == 0, "expecting length mod 3"
         return [_parser(payload[i : i + 6]) for i in range(0, len(payload), 6)]
 
@@ -1414,7 +1473,7 @@ def parser_2349(payload, msg) -> Optional[dict]:
             assert payload[6:8] != "03", f"{payload[6:8]} (0x00)"
         else:
             assert payload[6:8] == "03", f"{payload[6:8]} (0x01)"
-            result["minutes_remaining"] = int(payload[8:14], 16)
+            result["duration"] = int(payload[8:14], 16)
 
     if msg.len >= 13:
         if payload[14:] == "FF" * 6:
@@ -1426,11 +1485,11 @@ def parser_2349(payload, msg) -> Optional[dict]:
 
     # TODO: remove me...
     if TEST_MODE and msg.verb == W_:
-        KEYS = ("setpoint", "mode", "until")
+        KEYS = ("mode", "setpoint", "until", "duration")
         cmd = Command.set_zone_mode(
             msg.dst.id, payload[:2], **{k: v for k, v in result.items() if k in KEYS}
         )
-        assert cmd.payload == payload, f"test payload: {cmd.payload}"
+        assert cmd.payload == payload, f"{cmd.payload}, not {payload}"
     # TODO: remove me...
 
     return result
@@ -1508,12 +1567,18 @@ def parser_30c9(payload, msg) -> Optional[dict]:
 def parser_3120(payload, msg) -> Optional[dict]:
     #  I --- 34:136285 --:------ 34:136285 3120 007 0070B0000000FF  # every ~3:45:00!
     # RP --- 20:008749 18:142609 --:------ 3120 007 0070B000009CFF
+    #  I --- 37:258565 --:------ 37:258565 3120 007 0080B0010003FF
 
-    assert payload[:10] == "0070B00000", payload[:10]
+    assert payload[:2] == "00", payload[:2]
+    assert payload[2:4] in ("70", "80"), payload[2:4]
+    assert payload[4:6] == "B0", payload[4:6]
+    assert payload[6:8] in ("00", "01"), payload[6:8]
+    assert payload[8:10] == "00", payload[8:10]
+    assert payload[10:12] in ("00", "03", "9C"), payload[8:12]
     assert payload[12:] == "FF", payload[12:]
     return {
-        "unknown_1": payload[10:12],
         "unknown_0": payload[2:10],
+        "unknown_1": payload[10:12],
         "unknown_2": payload[12:],
     }
 
@@ -1577,51 +1642,117 @@ def parser_3150(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # ???
 def parser_31d9(payload, msg) -> Optional[dict]:
-    assert payload[2:4] in ("00", "06"), payload[2:4]
+    assert payload[2:4] in ("00", "06", "80"), payload[2:4]
     assert payload[4:6] == "FF" or int(payload[4:6], 16) <= 200, payload[4:6]
 
+    result = {
+        FanSwitch.FAN_RATE: _percent(payload[4:6]),  # NOTE: is 31DA/payload[38:40]
+        "_unknown_0": payload[2:4],
+    }
+
     if msg.len == 3:  # usu: I -->20: (no seq#)
-        return {
-            **_idx(payload[:2], msg),
-            FanSwitch.FAN_RATE: _percent(payload[4:6]),  # NOTE: is 31DA/payload[38:40]
-            "unknown_0": payload[2:4],
-        }
+        return result
 
     assert msg.len == 17, msg.len  # usu: I 30:-->30:, (or 20:) with a seq#!
     assert payload[6:8] == "00", payload[6:8]
     assert payload[8:32] in ("00" * 12, "20" * 12), payload[8:32]
+    assert payload[32:] == "00", payload[32:]
 
     return {
-        # **_idx(payload[:2], msg),
-        FanSwitch.FAN_RATE: _percent(payload[4:6]),  # NOTE: is 31D9/payload[4:6]
-        "unknown_0": payload[2:4],
-        "unknown_2": payload[6:8],
-        "unknown_3": payload[8:32],
-        "unknown_4": payload[32:],
+        **result,
+        "_unknown_2": payload[6:8],
+        "_unknown_3": payload[8:32],
+        "_unknown_4": payload[32:],
     }
 
 
 @parser_decorator  # UFC HCE80 (Nuaire humidity)
 def parser_31da(payload, msg) -> Optional[dict]:
-    assert msg.len == 29, msg.len  # usu: I CTL-->CTL
 
-    assert payload[2:10] in ("004007D0", "EF007FFF"), payload[2:10]  # also: "004007D0"
-    assert payload[12:30] == "EF7FFF7FFF7FFF7FFF", payload[12:30]
+    FAN_INFO = {
+        0x00: "off",
+        0x01: "speed 1",
+        0x02: "speed 2",
+        0x03: "speed 3",
+        0x04: "speed 4",
+        0x05: "speed 5",
+        0x06: "speed 6",
+        0x07: "speed 7",
+        0x08: "speed 8",
+        0x09: "speed 9",
+        0x0A: "speed 10",
+        0x0B: "speed 1 temporary override",
+        0x0C: "speed 2 temporary override",
+        0x0D: "speed 3 temporary override",
+        0x0E: "speed 4 temporary override",
+        0x0F: "speed 5 temporary override",
+        0x10: "speed 6 temporary override",
+        0x11: "speed 7 temporary override",
+        0x12: "speed 8 temporary override",
+        0x13: "speed 9 temporary override",
+        0x14: "speed 10 temporary override",
+        0x15: "away",
+        0x16: "absolute minimum",
+        0x17: "absolute maximum",
+        0x18: "auto",
+    }
+
+    def _percent(val, precision=0.5) -> Optional[float]:
+        if val in ("EF", "FF"):
+            return
+        if len(val) != 2:
+            raise TypeError(val)
+        result = int(val, 16)
+        if result > 100 / precision:
+            raise ValueError(result)
+        return result / 100 / precision
+
+    assert msg.len == 29, f"expected length 29, not {msg.len}"
+
+    assert payload[2:4] in ("00", "EF"), payload[2:4]
+    assert payload[4:6] in ("00", "40"), payload[4:6]
+    assert payload[6:10] in ("07D0", "7FFF"), payload[6:10]
+    assert payload[10:12] == "EF" or int(payload[10:12], 16) <= 100, payload[10:12]
+    assert payload[12:14] == "EF", payload[12:14]
+    assert payload[14:18] == "7FFF", payload[14:18]
+    assert payload[18:22] == "7FFF", payload[18:22]
+    assert payload[22:26] == "7FFF", payload[22:26]
+    assert payload[26:30] == "7FFF", payload[26:30]
+    assert payload[30:34] in ("0002", "F000", "F800", "F808", "7FFF"), payload[30:34]
     assert payload[34:36] == "EF", payload[34:36]
-    assert payload[42:44] == "00", payload[42:44]
+    assert payload[36:38] == "EF" or int(payload[36:38], 16) & 0x1F <= 0x18, payload[
+        36:38
+    ]
+    assert payload[38:40] in ("EF", "FF") or int(payload[38:40], 16) <= 200, payload[
+        38:40
+    ]
+    assert payload[40:42] in ("00", "EF", "FF"), payload[40:42]
+    # assert payload[42:46] == "0000", payload[42:46]
     assert payload[46:48] in ("00", "EF"), payload[46:48]
-    assert payload[48:] in ("EF7FFF7FFF", "EF7FFFFFFF"), payload[48:]
-
-    rh = int(payload[10:12], 16) / 100 if payload[10:12] != "EF" else None  # not /200!
+    assert payload[48:50] == "EF", payload[48:50]
+    assert payload[50:54] == "7FFF", payload[50:54]
+    assert payload[54:58] == "7FFF", payload[54:58]
 
     return {
-        # **_idx(payload[:2], msg),
-        "unknown_12c8": payload[4:6],
-        "unknown_1298": payload[6:10],
-        "relative_humidity": rh,  # [10:12]
-        "unknown_1": payload[30:38],
-        FanSwitch.FAN_RATE: _percent(payload[38:40]),  # NOTE: is 31D9/payload[4:6]
-        FanSwitch.BOOST_TIMER: int(payload[44:46], 16),
+        "air_quality": _percent(payload[2:4]),
+        "air_quality_base": int(payload[4:6], 16),  # NOTE: 12C8/payload[4:6]
+        "co2_level": _double(payload[6:10]),  # ppm NOTE: 1298/payload[2:6]
+        "indoor_humidity": _percent(payload[10:12], precision=1),  # TODO: 12A0?
+        "outdoor_humidity": _percent(payload[12:14], precision=1),
+        "exhaust_temperature": _double(payload[14:18], factor=100),
+        "supply_temperature": _double(payload[18:22], factor=100),
+        "indoor_temperature": _double(payload[22:26], factor=100),
+        "outdoor_temperature": _double(payload[26:30], factor=100),  # TODO: 1290?
+        "speed_cap": int(payload[30:34], 16),
+        "bypass_pos": _percent(payload[34:36]),
+        "fan_info": FAN_INFO[int(payload[36:38], 16) & 0x1F],
+        "exhaust_fan_speed": _percent(payload[38:40]),  # NOTE: 31D9/payload[4:6]
+        "supply_fan_speed": _percent(payload[40:42]),
+        "remaining_time": _double(payload[42:46]),  # mins NOTE: 22F3/payload[2:6]
+        "post_heat": _percent(payload[46:48]),
+        "pre_heat": _percent(payload[48:50]),
+        "supply_flow": _double(payload[50:54], factor=100),  # L/sec
+        "exhaust_flow": _double(payload[54:58], factor=100),  # L/sec
     }
 
 
@@ -1766,7 +1897,7 @@ def parser_3ef0(payload, msg) -> dict:
             assert int(payload[2:4], 16) <= 100, f"byte 1: {payload[2:4]}"
         assert payload[4:6] in ("10", "11"), f"byte 2: {payload[4:6]}"
         assert payload[8:10] in ("00", "01", "FF"), f"byte 4: {payload[8:10]}"
-        assert payload[10:12] == "FF", f"byte 5: {payload[10:12]}"
+        assert payload[10:12] in ("00", "FF"), f"byte 5: {payload[10:12]}"
 
     if msg.len > 6:  # <= 9: # for some OTB
         assert payload[-2:] in ("00", "64"), f"byte x: {payload[-2:]}"
