@@ -24,6 +24,9 @@ from serial import SerialException, serial_for_url
 from serial_asyncio import SerialTransport as SerialTransportAsync
 
 from .command import (
+    ARGS,
+    DEAMON,
+    FUNC,
     QOS_MAX_BACKOFF,
     QOS_RX_TIMEOUT,
     QOS_TX_RETRIES,
@@ -310,10 +313,10 @@ class PacketProtocolBase(asyncio.Protocol):
     def is_wanted(self, src_addr, dst_addr) -> bool:
         """Parse the packet, return True if the packet is not to be filtered out."""
         pkt_addrs = {src_addr, dst_addr}
-        if any(d.type == "18" for d in pkt_addrs):
+        if any(d.type == "18" for d in pkt_addrs):  # TODO: use GWY's full addr
             return True
         wanted = not self._include or any(d.id in self._include for d in pkt_addrs)
-        return wanted and all(d.id not in self._exclude for d in pkt_addrs)
+        return wanted or not all(d.id not in self._exclude for d in pkt_addrs)
 
     @staticmethod
     def _normalise(pkt_line: str) -> str:
@@ -699,6 +702,14 @@ class PacketProtocolQos(PacketProtocolBase):
                 message,
             )
 
+        def _expired_cmd(cmd):
+            hdr, callback = cmd.tx_header, cmd.callback
+            if callback and not callback.get("expired"):
+                # see also: MsgTransport._pkt_receiver()
+                _LOGGER.error("PktProtocolQos.send_data(%s): Expired callback", hdr)
+                callback[FUNC](False, *callback.get(ARGS, tuple()))
+                callback["expired"] = not callback.get(DEAMON, False)
+
         if self._gwy.config[DISABLE_SENDING]:
             raise RuntimeError("Sending is disabled")
 
@@ -746,13 +757,18 @@ class PacketProtocolQos(PacketProtocolBase):
                 self._timeouts(dt.now())
                 await self._send_data(bytes(f"{cmd}\r\n".encode("ascii")))
                 _logger_send(
-                    _LOGGER.warning,
+                    _LOGGER.debug,
                     f"RE-SENT ({self._tx_retries}/{self._tx_retry_limit})",
                 )  # TODO: should be info/debug
 
             else:
                 if self._qos_cmd.code != "7FFF":  # HACK: why expired when shouldn't
-                    _logger_send(_LOGGER.error, "EXPIRED")
+                    _logger_send(
+                        _LOGGER.warning,
+                        f"EXPIRED ({self._tx_retries}/{self._tx_retry_limit})",
+                    )
+                    _expired_cmd(self._qos_cmd)
+
                 self._qos_lock.acquire()
                 self._qos_cmd = None
                 self._qos_lock.release()
